#!/usr/local/bin/wish8.2
package require gd

proc textwheel { image text x y {increment 15} { color {255 0 0} } {font {Times 16} } {antialias true } } {
    # Since text fonts are supposed to (from the canvas) be measured from the UL corner, we must
    # adjust the center point to be in the center of the "linewheel" by moving it 1/2 of a font height
    # perpendicular to the angle
  for { set i 0.0 } { $i < 360.0 } { set i [ expr $i + $increment ] } {
    if { $antialias == "true" } { set extra "-antialias" } else { set extra "-noantialias" }
    lappend outtxt [
    	gd text $image $x $y -text $text -rotate $i -anchor nw -justify left -font $font -fill $color $extra -debug -single
    ]
  }
  return $outtxt
}

proc linewheel { image x y {increment 15} {color {0 255 0} } {offset 20 } { length 100 } } {
  for { set i 0.0 } { $i < 360.0 } { set i [ expr $i + $increment ] } {
    set angle [ expr $i * ( 2.0 * 3.14159265) / 360.0 ]
    set x0 [ expr cos($angle)*$offset + $x ]
    set x1 [ expr cos($angle)*($offset+$length) + $x ]
    set y0 [ expr sin($angle)*($offset) + $y]
    set y1 [ expr sin($angle)*($offset+$length) + $y]
    gd line $image $x0 $y0 $x1 $y1 -fill $color
  }
}

proc testimage { file w h bg text font angle fc lc xtra } {
  set image [ gd open -width $w -height $h -background $bg ]
  gd rectangle $image 1 1 [ expr $w - 1 ] [ expr $h - 1 ] -outline { 0 0 255 } -fill { 0 0 0 }
  set output [
    textwheel $image ".....Hello, there!" [ expr $w / 2 ] [ expr $h / 2 ] $angle $fc $font $xtra
  ]
  linewheel $image [ expr $w / 2 ] [ expr $h / 2 ] $angle
  gd save $image -file $file
  gd close $image
  return $output
}

lappend output [
  testimage fw1.png 400 400 {0 0 0} ".....Hello, there!" {Arial 16} 30 {255 192 192} { 0 255 255 } true
]
lappend output [
testimage fw2.png 400 400 {0 0 0} ".....Hello, there!" {Times 16} 20 {255 192 192} { 0 255 255 } true
]
lappend output [
testimage fw3.png 400 400 {0 0 0} ".....Hello, there!" {Arial 16} 45 {255 192 192} { 0 255 255 } false
]
lappend output [
testimage fw4.png 400 400 {0 0 0} ".....Hello, there!" {Times 16} 10.0 {255 192 192} { 0 255 255 } false
]


if { $tcl_interactive == 0 } {
  exit
} else {
  puts [ join $output \n ]
}

