
/* Replace the GD string functions with ones that respect newlines */
#define GD_IMAGESTRING_SUPPRESS_NEWLINE 1
static void gdImageStringEx(gdImagePtr im, gdFontPtr f, 
	int x, int y, unsigned char *s, int color, int formatflags)
{
  int xstart = x;
  int i;
  int l;
  l = strlen((char *) s);
  for (i=0; (i<l); i++) {
    if ( (formatflags & GD_IMAGESTRING_SUPPRESS_NEWLINE ) == 0 )
    {
      if ( s[i] == '\n' ) /* Treat like CR/LF */
      {
        x = xstart;
        y += f->h;
      }
      else if (s[i] == '\r') /* Ignore */
        continue;
    }

    gdImageChar(im, f, x, y, s[i], color);
    x += f->w;
  }
}

static void gdImageStringUpEx(gdImagePtr im, gdFontPtr f, 
	int x, int y, unsigned char *s, int color, int formatflags)
{
  int ystart = y;
  int i;
  int l;
  l = strlen((char *) s);
  for (i=0; (i<l); i++) {
    if ( (formatflags & GD_IMAGESTRING_SUPPRESS_NEWLINE ) == 0 )
    {
      if ( s[i] == '\n' ) /* Treat like CR/LF */
      {
        y = ystart;
        x += f->h;
      }
      else if (s[i] == '\r') /* Ignore */
        continue;
    }
    gdImageCharUp(im, f, x, y, s[i], color);
    y -= f->w;
  }
}

static void gdImagePolygonEx(gdImagePtr im, gdPointPtr p, int n, int c, int thick)
{
	if (!n) {
		return;
	}
	gdImageLineEx(im, p->x, p->y, p[n-1].x, p[n-1].y, c, thick);
	gdImageOpenPolygonEx(im, p, n, c, thick);
}	
	
static void gdImageOpenPolygonEx(gdImagePtr im, gdPointPtr p, int n, int c, int thick)
{
	int i;
	int lx, ly;
	if (!n) {
		return;
	}
	lx = p->x;
	ly = p->y;
	for (i=1; (i < n); i++) {
		p++;
		gdImageLineEx(im, lx, ly, p->x, p->y, c, thick );
		lx = p->x;
		ly = p->y;
	}
}	

static void gdImageLineEx (gdImagePtr im, int x1, int y1, int x2, int y2, int color, int thick)
{
	int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
	int wid;
	int w, wstart;

	dx = abs(x2-x1);
	dy = abs(y2-y1);
	if (dy <= dx) {
		/* More-or-less horizontal. use wid for vertical stroke */
		wid = (int)(thick / cos ( atan2(dy, dx) ));
		if (wid==0) wid=1;

		d = 2*dy - dx;
		incr1 = 2*dy;
		incr2 = 2 * (dy - dx);
		if (x1 > x2) {
			x = x2;
			y = y2;
			ydirflag = (-1);
			xend = x1;
		} else {
			x = x1;
			y = y1;
			ydirflag = 1;
			xend = x2;
		}

		/* Set up line thickness */
		wstart = y - wid/2;
		for ( w = wstart; w < wstart + wid; w++ )
		  gdImageSetPixel(im, x, w, color);
		  
		if (((y2 - y1) * ydirflag) > 0) {
			while (x < xend) {
				x++;
				if (d <0) {
					d+=incr1;
				} else {
					y++;
					d+=incr2;
				}
				wstart = y - wid/2;
				for ( w = wstart; w < wstart + wid; w++)
				  gdImageSetPixel(im, x, w, color);
			}
		} else {
 			while (x < xend) {
				x++;
				if (d <0) {
					d+=incr1;
				} else {
					y--;
					d+=incr2;
				}
				wstart = y - wid / 2;
				for ( w=wstart; w < wstart+wid; w++)
				  gdImageSetPixel(im, x, w, color);
			}
		}		
	} else {
  		/* More-or-less vertical. use wid for horizontal stroke */
		wid = (int)(thick / sin ( atan2(dy, dx) ));
		if (wid == 0 ) wid=1;

		d = 2*dx - dy;
		incr1 = 2*dx;
		incr2 = 2 * (dx - dy);
		if (y1 > y2) {
			y = y2;
			x = x2;
			yend = y1;
			xdirflag = (-1);
		} else {
			y = y1;
			x = x1;
			yend = y2;
			xdirflag = 1;
		}

		/* Set up line thickness */
		wstart = x - wid/2;
		for ( w = wstart; w < wstart + wid; w++ )
		  gdImageSetPixel(im, w, y, color);

		if (((x2 - x1) * xdirflag) > 0) {
			while (y < yend) {
				y++;
				if (d <0) {
					d+=incr1;
				} else {
					x++;
					d+=incr2;
				}
				wstart = x - wid/2;
				for ( w=wstart; w < wstart + wid; w++)
				  gdImageSetPixel(im, w, y, color);
			}
		} else {
			while (y < yend) {
				y++;
				if (d <0) {
					d+=incr1;
				} else {
					x--;
					d+=incr2;
				}
				wstart = x - wid/2;
				for ( w = wstart; w < wstart+wid; w++)
				  gdImageSetPixel(im, w, y, color);
			}
		}
	}
}


#ifndef HAVE_LSQRT
/* If you don't have a nice square root function for longs, you can use
** this hack
*/
long lsqrt(long n)
{
  long result = (long)sqrt((double)n);
  return result;
}
#endif

static void gdImageEllipseEx (gdImagePtr im, int cx, int cy, int w, int h, int color, int thick)
{
  long d, b_sq, b_sq_4, b_sq_6;
  long a_sq, a_sq_4, a_sq_6;
  int x, y, switchem;
  long lsqrt(long);
  int pix, half, pstart;

  half = thick/2;
  w /= 2; /* ImageArc uses diameter, not radius */
  h /= 2;
      
  d = 2 * (long) h * h + (long) w * w - 2 * (long)w * w * h;
  b_sq = (long)h * h;
  b_sq_4 = 4 * (long) h * h;
  b_sq_6 = 6 * (long) h * h;
  a_sq = (long) w * w;
  a_sq_4 = 4 * (long)w * w;
  a_sq_6 = 6 * (long)w * w;

  x = 0;
  y = -h;
  switchem = a_sq / lsqrt( a_sq + b_sq );

  while ( x <= switchem )
  {
    pstart = y - half;
    for (pix = pstart; pix < pstart + thick; pix++)
    {
      gdImageSetPixel(im, cx+x, cy+pix, color);
      gdImageSetPixel(im, cx-x, cy+pix, color);
      gdImageSetPixel(im, cx+x, cy-pix, color);
      gdImageSetPixel(im, cx-x, cy-pix, color);
    }
    if ( d < 0 )
      d += b_sq_4 * x++ + b_sq_6;
    else
      d += b_sq_4*x++ + b_sq_6 + a_sq_4*(++y);
  }

  /* Middlesplat!
  ** Go a little further if the thickness is not nominal...
  */
  if ( thick > 1 )
  {
    int xp = x;
    int yp = y;
    int dp = d;
    int thick2 = thick + 2;
    int half2 = half + 1;

    while ( xp <= switchem + half )
    {
      pstart = yp - half2;
      for (pix = pstart; pix < pstart + thick2; pix++)
      {
        gdImageSetPixel(im, cx+xp, cy+pix, color);
        gdImageSetPixel(im, cx-xp, cy+pix, color);
        gdImageSetPixel(im, cx+xp, cy-pix, color);
        gdImageSetPixel(im, cx-xp, cy-pix, color);
      }
      if ( dp < 0 )
        dp += b_sq_4 * xp++ + b_sq_6;
      else
        dp += b_sq_4*xp++ + b_sq_6 + a_sq_4*(++yp);
    }
  }
  
  d += -2*(long)b_sq + 2*(long)a_sq - 2*(long)b_sq*(x-1) + 2*(long)a_sq*(y-1);

  while ( y <= 0 )
  {
    pstart = x - half;
    for (pix = pstart; pix < pstart + thick; pix++)
    {
      gdImageSetPixel(im, cx+pix, cy+y, color);
      gdImageSetPixel(im, cx-pix, cy+y, color);
      gdImageSetPixel(im, cx+pix, cy-y, color);
      gdImageSetPixel(im, cx-pix, cy-y, color);
    }
    
    if (d < 0)
      d += a_sq_4*y++ + a_sq_6 + b_sq_4*(++x);
    else
      d += a_sq_4*y++ + a_sq_6;
  }
}

/* s and e are integers modulo 360 (degrees), with 0 degrees
  being the rightmost extreme and degrees changing clockwise.
  cx and cy are the center in pixels; w and h are the horizontal 
  and vertical diameter in pixels. Nice interface, but slow, since
  I don't yet use Bresenham (I'm using an inefficient but
  simple solution with too much work going on in it; generalizing
  Bresenham to ellipses and partial arcs of ellipses is non-trivial,
  at least for me) and there are other inefficiencies (small circles
  do far too much work). */

static int lineside ( int x, int y, int dx, int dy, int x0, int y0 )
{
  return ( y - y0) * dx - dy * ( x - x0 ) < 0;
}

static void gdImageArcEx(gdImagePtr im, int cx, int cy, int w, int h, int s, int e, int color, int thick)
{
	double w2, h2;
	int xr0, xr1, yr0, yr1; /* starting and ending coordinates */
	int dx, dy;             /* slope of the resulting line */
        long d, b_sq, b_sq_4, b_sq_6;
        long a_sq, a_sq_4, a_sq_6;
        int x, y, switchem;
        long lsqrt(long);
        int pix, half, pstart;

        half = thick/2;

	w2 = w / 2.0;
	h2 = h / 2.0;
	while (s > 360) s -= 360;
	while (s < 0 ) s += 360;
	while (e < s ) e += 360;

	if ( e - s >= 359 )
	{ 
          gdImageEllipseEx (im, cx, cy, w, h, color, thick);
	  return;
	}

	xr0 = xr1 = cx;
	yr0 = yr1 = cy;
	xr0 += (int)(w2 * cos ( (s * 2.0 * 3.14159265) / 360.0 ));
	yr0 += (int)(h2 * sin ( (s * 2.0 * 3.14159265) / 360.0 ));
	xr1 += (int)(w2 * cos ( (e * 2.0 * 3.14159265 ) / 360.0 ));
	yr1 += (int)(h2 * sin ( (e * 2.0 * 3.14159265 ) / 360.0 ));
        dx = xr1 - xr0;
        dy = yr1 - yr0;

	/* From here it should be just like the ellipse, but qualified to be on 
	** ONE side of the chord from xr0,yr0 to xr1, yr1 (positive)
	*/
        d =      (long)(2 * (long) h2 * h2 + (long) w2 * w2 - 2 * (long)w2 * w2 * h2);
        b_sq =   (long)(h2 * h2);
        b_sq_4 = 4 * (long) (h2 * h2);
        b_sq_6 = 6 * (long) (h2 * h2);
        a_sq =   (long) (w2 * w2);
        a_sq_4 = 4 * (long)(w2 * w2);
        a_sq_6 = 6 * (long)(w2 * w2);

        x = 0;
        y = (int)-h2;
        switchem = a_sq / lsqrt( a_sq + b_sq );

        while ( x <= switchem )
        {
          pstart = y - half;
          for (pix = pstart; pix < pstart + thick; pix++)
          {
            int xx, yy;

            xx = cx+x; yy = cy+pix;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx - x;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx+x; yy = cy-pix;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx-x;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);
          }
          if ( d < 0 )
            d += b_sq_4 * x++ + b_sq_6;
          else
            d += b_sq_4*x++ + b_sq_6 + a_sq_4*(++y);
        }

        /* Middlesplat!
        ** Go a little further if the thickness is not nominal...
        */
        if ( thick > 1 )
        {
          int xp = x;
          int yp = y;
          int dp = d;
          int thick2 = thick + 2;
          int half2 = half + 1;
      
          while ( xp <= switchem + half )
          {
            pstart = yp - half2;
            for (pix = pstart; pix < pstart + thick2; pix++)
            {
              int xx, yy;

              xx = cx+xp; yy = cy+pix;
              if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
                gdImageSetPixel(im, xx, yy, color);

              xx = cx - xp;
              if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
                gdImageSetPixel(im, xx, yy, color);

              xx = cx+xp; yy = cy-pix;
              if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
                gdImageSetPixel(im, xx, yy, color);

              xx = cx-xp;
              if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
                gdImageSetPixel(im, xx, yy, color);
            }
            if ( dp < 0 )
              dp += b_sq_4 * xp++ + b_sq_6;
            else
              dp += b_sq_4*xp++ + b_sq_6 + a_sq_4*(++yp);
          }
        }
  
        d += -2*(long)b_sq + 2*(long)a_sq - 2*(long)b_sq*(x-1) + 2*(long)a_sq*(y-1);

        while ( y <= 0 )
        {
          pstart = x - half;
          for (pix = pstart; pix < pstart + thick; pix++)
          {
            int xx, yy;

            xx = cx+pix; yy = cy+y;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx - pix;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx+pix; yy = cy-y;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);

            xx = cx-pix;
            if ( lineside ( xx, yy, dx, dy, xr0, yr0 ) )
              gdImageSetPixel(im, xx, yy, color);
          }
    
          if (d < 0)
            d += a_sq_4*y++ + a_sq_6 + b_sq_4*(++x);
          else
            d += a_sq_4*y++ + a_sq_6;
        }
}


static void gdImageRectangleEx(gdImagePtr im, int x1, int y1, int x2, int y2, int color, int thick)
{
	int x1h=x1, x1v=x1, y1h=y1, y1v=y1, x2h=x2, x2v=x2, y2h=y2, y2v=y2;
	if (thick > 1)
	{
	  int half = thick/2;
	  int half1 = thick - half;

	  if ( y1 < y2 )
	  {
	    y1v = y1h - half;
            y2v = y2h + half1 - 1;
	  }
	  else
	  {
	    y1v = y1h + half1 - 1;
            y2v = y2h - half;
	  }
	}

	gdImageLineEx(im, x1h, y1h, x2h, y1h, color, thick);		
	gdImageLineEx(im, x1h, y2h, x2h, y2h, color, thick);		
	gdImageLineEx(im, x1v, y1v, x1v, y2v, color, thick);
	gdImageLineEx(im, x2v, y1v, x2v, y2v, color, thick);
}

static void dashedSet(gdImagePtr im, int x, int y, int color,
	int *onP, int *dashStepP, int wid, int vert);

void gdImageDashedLineEx(gdImagePtr im, int x1, int y1, int x2, int y2, int color, int thick)
{
	int dx, dy, incr1, incr2, d, x, y, xend, yend, xdirflag, ydirflag;
	int dashStep = 0;
	int on = 1;
        int wid;
        int vert;

	dx = abs(x2-x1);
	dy = abs(y2-y1);
	if (dy <= dx) {
		/* More-or-less horizontal. use wid for vertical stroke */
		wid = (int)(thick / sin ( atan2(dy, dx) ));
		vert=1;

		d = 2*dy - dx;
		incr1 = 2*dy;
		incr2 = 2 * (dy - dx);
		if (x1 > x2) {
			x = x2;
			y = y2;
			ydirflag = (-1);
			xend = x1;
		} else {
			x = x1;
			y = y1;
			ydirflag = 1;
			xend = x2;
		}
		dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
		if (((y2 - y1) * ydirflag) > 0) {
			while (x < xend) {
				x++;
				if (d <0) {
					d+=incr1;
				} else {
					y++;
					d+=incr2;
				}
				dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
			}
		} else {
			while (x < xend) {
				x++;
				if (d <0) {
					d+=incr1;
				} else {
					y--;
					d+=incr2;
				}
		                dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
			}
		}		
	} else {
		/* More-or-less vertical. use wid for horizontal stroke */
		wid = (int)(thick / sin ( atan2(dy, dx) ));
		vert = 0;

		d = 2*dx - dy;
		incr1 = 2*dx;
		incr2 = 2 * (dx - dy);
		if (y1 > y2) {
			y = y2;
			x = x2;
			yend = y1;
			xdirflag = (-1);
		} else {
			y = y1;
			x = x1;
			yend = y2;
			xdirflag = 1;
		}
                dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
		if (((x2 - x1) * xdirflag) > 0) {
			while (y < yend) {
				y++;
				if (d <0) {
					d+=incr1;
				} else {
					x++;
					d+=incr2;
				}
                                dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
			}
		} else {
			while (y < yend) {
				y++;
				if (d <0) {
					d+=incr1;
				} else {
					x--;
					d+=incr2;
				}
                                dashedSet(im, x, y, color, &on, &dashStep, wid, vert);
			}
		}
	}
}

static void dashedSet(gdImagePtr im, int x, int y, int color,
	int *onP, int *dashStepP, int wid, int vert)
{
	int dashStep = *dashStepP;
	int on = *onP;
	int w, wstart;

	dashStep++;
	if (dashStep == gdDashSize) {
		dashStep = 0;
		on = !on;
	}
	if (on) {
          if (vert) { 
	    wstart = y - wid / 2; 
	    for (w=wstart; w < wstart+wid;w++) 
	      gdImageSetPixel(im, x, w, color); 
	  }
	  else 
	  { 
	    wstart = x - wid / 2; 
	    for (w=wstart; w < wstart+wid;w++) 
	      gdImageSetPixel(im, w, y, color); 
	  }
	}
	*dashStepP = dashStep;
	*onP = on;
}
	


