package require gd

if { $tcl_interactive == 0 } {
  proc P { msg } { }
} else {
  proc P { msg } {
    puts $msg
    update idletasks
    after 100
  }
}

P "Opening demoin.png"

# Let the type default
set imgpath demoin.png
set im [ gd open -file $imgpath ]

P "Configuring $im"
# Now calculate the size of the drawing
set sizeinfo [ gd config $im -size ]
set diminfo [ lindex [ lindex $sizeinfo 0 ] 1 ]
set x [ lindex $diminfo 0]
set y [ lindex $diminfo 1]

# Determine how many existing color indices there are
set colinfo [ gd config $im -colormap ]
set numcol [ llength [ lindex [ lindex $colinfo 0 ] 1 ]  ]

# Now draw some stuff

P "Drawing lines on $im"
# Draw a big X
gd line $im 5 5 [expr $x - 5] [expr $y - 5] -fill "255 128 0"
gd line $im 5 [expr $y - 5] [expr $x - 5] 5 -fill "128 255 0"

P "Drawing a rectangle on $im"
# Draw a rectangle in color 1
gd rectangle $im 10 10 [ expr $x - 10 ] [ expr $y - 10 ] -outline "#1"

P "Filling in areas on $im"
# fill in the outside of the rectangle
gd fill $im 2 2 -outline "#1" -fill "255 128 0"

# Draw a rectangle and fill it
gd rectangle $im 20 20 [ expr $x - 20 ] [ expr $y - 20 ] -outline "255 0 255" -fill "0 0 255"

P "Drawing polygons on $im"
# Draw a polygon, outline it with a "style" line and fill it with a pattern
set fillim [ gd open -width 8 -height 8 ]
gd line $fillim 0 0 0 9 -fill "0 0 0"
gd line $fillim 2 0 2 9 -fill "255 0 0"
gd line $fillim 4 0 4 9 -fill "255 0 0"
gd line $fillim 6 0 6 9 -fill "255 0 0"
gd line $fillim 8 0 8 9 -fill "255 0 0"
# gd config $fillim -transparent #0
#gd rectangle $fillim 0 0 9 9 -outline "255 0 0"
#gd line $fillim 0 0 9 9 -fill "255 0 0"
#gd line $fillim 0 9 9 0 -fill "255 0 0"
#gd rectangle $fillim 4 4 5 5 -fill "0 0 255"

P "Drawing a tiled line"
gd line $im [ expr $x - 30 ]  10 [expr $x - 30 ] [expr $y - 30] -fill @$fillim

set midx [ expr $x / 2 ]
set midy [ expr $y / 2 ]
gd polygon $im 30 30 $midx 30 $midx $midy [ expr $midx + 10 ] [ expr $midy + 10 ] $midx [expr $y - 30] -outline "=0 0 1 1 1 1" -fill %$fillim

# Put in some horizontal text
gd text $im 10 [ expr $y / 2 ] -text "Horizontal text" -fill "255 255 0" -orientation horizontal

# Put in some vertical text
gd text $im 10 [ expr $y - 10] -text "Vertical text" -font small -fill "255 0 255" -orientation vertical

# GIF code removed from distribution
# P "Saving as xxx0.gif"
# Write the resulting image as xxx0.gif
# gd close $im -save -file gdtest.gif 

P "Saving as xxx0.png"
# set im [gd open -file xxx0.gif]
gd close $im -save -file gdtest.png

P "Reading in demoin.png and enlarging to 640x480"
# Read in the file and enlarge it
set im [ gd open -file $imgpath -width 640 -height 480 ]

P "Adding arcs to $im"
# Put in all kinds of arcs
gd arc $im 10 10 310 240 -start 30 -extent 120 -style pieslice -outline "255 255 255"
gd arc $im 330 10 630 240 -start 30 -extent 120 -style chord -outline  "255 255 255"
gd arc $im 10 250 310 470 -start 30 -extent 120 -style arc -outline  "255 255 255"

P "Adding text to $im"
# Put in all sizes of text
gd text $im 330 240 -text "tiny text" -font tiny -fill  "255 255 255"
gd text $im 330 250 -text "small text" -font small -fill "255 255 255"
gd text $im 330 270 -text "large text*" -font large -fill "255 255 255"
gd text $im 330 300 -text "giant" -font giant -fill    "255 255 255"
gd text $im 330 330 -text "medium bold" -font medium -fill "255 255 255"

P "Putting \"fat\" lines on $im"
# Put in a "fat" line
set brush [ gd open -width 4 -height 1 ]
gd line $brush 0 0 3 0 -fill "0 0 0"
gd line $brush 0 0 3 0 -fill "255 50 50"
gd config $brush -transparent "#0"

gd line $im 5 5 600 400 -fill @$brush

P "Putting arrowed lines on $im"
# Put in arrowed lines
gd line $im 100 100 100 200 -arrow both -fill {255 0 0}
gd line $im 120 100 120 200 -arrow both -fill {255 0 0} -arrowshape {8 12 4}
gd line $im 140 100 140 200 -arrow both -fill {255 0 0} -width 2 -arrowshape {10 12 4}
gd line $im 160 100 160 200 -arrow both -fill {255 0 0} -width 3 -arrowshape {12 12 4}
gd line $im 180 100 180 200 -arrow both -fill {255 0 0} -width 5 -arrowshape {14 12 4}

# GIF code removed from distribution
# P "Writing xxx1.gif"
# Save the result as xxx1.gif
# gd close $im -save -file gdtest1.gif
gd close $im -save -file gdtest1.png

P "Copying a a png"
# Convert demoin to a png so we can use it....
set im [gd open -file $imgpath]
gd close $im -save -file demoin2.png

P "Reading in demoin.png and enlarging to 640x480"
# Read in the file and enlarge it
set im [ gd open -file demoin.png -width 640 -height 480 ]

P "Putting arcs on $im"
# Put in all kinds of arcs
gd arc $im 10 10 310 240 -start 30 -extent 120 -style pieslice -outline "255 255 255"
gd arc $im 330 10 630 240 -start 30 -extent 120 -style chord -outline  "255 255 255"
gd arc $im 10 250 310 470 -start 30 -extent 120 -style arc -outline  "255 255 255"

P "Putting filled arcs on $im"
# Now fill the arcs....
gd arc $im 20 20 300 230 -start 30 -extent 120 -style pieslice -outline "255 255 255" -fill "192 192 128"
gd arc $im 340 20 620 230 -start 30 -extent 120 -style chord -outline  "255 255 255" -fill "192 192 128"

# Currently, filling an arc (and some other shapes) with tiles causes a stack overflow.
# If this is still the case, fill with khaki.
if [ catch {
  gd arc $im 20 260 300 460 -start 30 -extent 120 -style arc -outline  "255 255 255" -fill "192 192 128"
           } err ] {
  gd arc $im 20 260 300 460 -start 30 -extent 120 -style arc -outline  "255 255 255" -fill "192 192 128"
}

P "Putting ellipses into $im"
# Put in ovals to surround the arcs
gd oval $im 8 8 312 242 -outline "255 255 255"
gd oval $im 328 8 632 242 -outline "255 255 255"
gd oval $im 8 248 312 472 -outline "255 255 255"

P "Putting text into $im"
# Put in all sizes of text
gd text $im 330 240 -text "tiny text" -font tiny -fill  "255 255 255"
gd text $im 330 250 -text "small text" -font small -fill "255 255 255"
gd text $im 330 270 -text "large text*" -font large -fill "255 255 255"
gd text $im 330 300 -text "giant" -font giant -fill    "255 255 255"
gd text $im 330 330 -text "medium bold" -font medium -fill "255 255 255"

P "Putting European characters from font into $im"
gd text $im 30 40 -text "medium bold with " -font medium -rotate 15 -fill "255 255 200"

P "Putting truetype font into $im"
gd text $im 330 400 -text "Truetype Times 16 with " -font {Times 16} -rotate 15 -fill "255 255 200"

# Put in a "double" line
set brush [ gd open -width 4 -height 4 ]
gd rectangle $brush 0 0 3 3 -fill "0 0 0"
gd line $brush 0 2 1 3 -fill "50 255 50"
gd line $brush 2 0 3 1 -fill "255 50 50"
gd config $brush -transparent "#0"

P "Putting in a brushed line into $im"
gd line $im 5 5 600 400 -fill @$brush
gd line $im 5 400 600 5 -fill @$brush

P "Saving as xxx2.png"
# Save the result as gdtest2.png
gd close $im -save -file gdtest2.png
P "Done"

if { $tcl_interactive == 0 } {
  exit
}
